/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten and Kirsten ten Tusscher


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "Arrays.hh"
#include "Auxin_params.hh"
#include "Basic_params.hh"
#include "Functions.hh"
#include "Param_config.hh"
#include "Plant_layout.hh"

#include <math.h>


void dsolvetridiag(int ni)
{
  /**
   * ni - number of equations
   * a - sub-diagonal (means it is the diagonal below the main diagonal) -- indexed from 1..n-1
   * b - the main diagonal
   * c - sup-diagonal (means it is the diagonal above the main diagonal) -- indexed from 0..n
   * rhs - right part
   * x - the answer
   */
  int i;
  double mm;
  
  for (i = 1; i < ni; i++)
    {
      mm = a[i]/b[i-1];
      b[i] = b[i] - mm*c[i-1];
      rhs[i] = rhs[i] - mm*rhs[i-1];
    }

  res[ni-1] = rhs[ni-1]/b[ni-1];
  
  for (i = ni - 2; i >= 0; i--)
    res[i]=(rhs[i]-c[i]*res[i+1])/b[i];
  
  for(i=0;i<ni;i++)
    rhs[i]=res[i];
}


void UpdateAuxin(int t)
{

  int i,j;
  static int Direction=0;
  int q;
  double func;
  if(Direction==0)//Implicit integration in j direction
    {             //Explicit integration in i direction
      Direction=1;
      
      //do one direction implicit one explicit
      for(i=0;i<n;i++)
	{
	  for(j=0;j<m;j++)
	    {
	      //Compute coefficients for implicit diffusion and fluxes
	      //diffusion only

	      if(Tc[i][j]==3)
		{
		  a[j]=0;
		  c[j]=0;
		  b[j]=1;
		  rhs[j]=0;
		}
	      else   
		{
		  a[j]=-DiffAuxC[i][j][3];//j-1
		  c[j]=-DiffAuxC[i][j][2];//j+1 
		  b[j]=1-(a[j]+c[j]);//j 
		  if(Tc[i][j]==0)
		    {
		      //influx
		      b[j]+=-0.5*tstep*(Inf[i][j][2]+Inf[i][j][3]);
		      //efflux
		      a[j]+=-0.5*tstep*Eff[i][j][3];
		      c[j]+=-0.5*tstep*Eff[i][j][2];
		    }
		  else if(Tc[i][j]==2)
		    {
		      //influx
		      a[j]+=-0.5*tstep*Inf[i][j][3];
		      c[j]+=-0.5*tstep*Inf[i][j][2];
		      //efflux
		      b[j]+=-0.5*tstep*(Eff[i][j][2]+Eff[i][j][3]);
		    }
		  
		  //explicit diffusion and fluxes
		  rhs[j]=Caux1[i][j]+DiffAuxC[i][j][0]*(Caux1[i+1][j]-Caux1[i][j])
		    +DiffAuxC[i][j][1]*(Caux1[i-1][j]-Caux1[i][j]);
		  if(Tc[i][j]==0)
		    {
		      rhs[j]+=0.5*tstep*(Eff[i][j][0]*Caux1[i+1][j]+Eff[i][j][1]*Caux1[i-1][j]);
		      rhs[j]+=0.5*tstep*((Inf[i][j][0]+Inf[i][j][1])*Caux1[i][j]);
		    }
		  else if(Tc[i][j]==2)
		    {
		      rhs[j]+=0.5*tstep*((Eff[i][j][0]+Eff[i][j][1])*Caux1[i][j]);
		      rhs[j]+=0.5*tstep*(Inf[i][j][0]*Caux1[i+1][j]+Inf[i][j][1]*Caux1[i-1][j]); 
		    }
		}//else: not Tc[i][j]==3
	    }
	  //Compute implicit diffusion and use this to update rhs
	  //Use new rhs to update volt2
	  dsolvetridiag(m); 
	  for(j=0;j<m;j++)
	    {
	      if(Tc[i][j]!=3)
		{
		  func=0.5*tstep*(paux[i][j]-daux[i][j]*Caux1[i][j]);
		  Cauxint[i][j]=rhs[j]+func;
		}
	    }
	}
      
    }
  if(Direction==1)//Implicit integration in i direction
    {             //Explicit integration in j direction
      Direction=0;
      
      //do one direction implicit one explicit
      for(j=0;j<m;j++)
	{
	  for(i=0;i<n;i++)
	    {
	      //Compute coefficients for implicit diffusion and fluxes
	      //diffusion only
	      if(Tc[i][j]==3)
		{
		  a[i]=0;
		  c[i]=0;
		  b[i]=1;
		  rhs[i]=0;
		}
	      else
		{
		  a[i]=-DiffAuxC[i][j][1];//i-1 direction
		  c[i]=-DiffAuxC[i][j][0];//i+1 direction
		  b[i]=1-(a[i]+c[i]);//i;
		  if(Tc[i][j]==0)
		    {
		      //influx
		      b[i]+=-0.5*tstep*(Inf[i][j][0]+Inf[i][j][1]);
		      //efflux
		      a[i]+=-0.5*tstep*Eff[i][j][1];
		      c[i]+=-0.5*tstep*Eff[i][j][0];
		    }
		  else if(Tc[i][j]==2)
		    {
		      //influx
		      a[i]+=-0.5*tstep*Inf[i][j][1];
		      c[i]+=-0.5*tstep*Inf[i][j][0];
		      //efflux
		      b[i]+=-0.5*tstep*(Eff[i][j][0]+Eff[i][j][1]);
		    }
		  
		  
		  //explicit diffusion and fluxes
		  rhs[i]=Cauxint[i][j]+DiffAuxC[i][j][2]*(Cauxint[i][j+1]-Cauxint[i][j])
		    +DiffAuxC[i][j][3]*(Cauxint[i][j-1]-Cauxint[i][j]);
		  if(Tc[i][j]==0)
		    {
		      rhs[i]+=0.5*tstep*(Eff[i][j][2]*Cauxint[i][j+1]+Eff[i][j][3]*Cauxint[i][j-1]);
		      rhs[i]+=0.5*tstep*((Inf[i][j][2]+Inf[i][j][3])*Cauxint[i][j]);
		    }
		  else if(Tc[i][j]==2)
		    {
		      rhs[i]+=0.5*tstep*((Eff[i][j][2]+Eff[i][j][3])*Cauxint[i][j]);
		      rhs[i]+=0.5*tstep*(Inf[i][j][2]*Cauxint[i][j+1]+Inf[i][j][3]*Cauxint[i][j-1]);
		    }
		}
	    }
	  //Compute implicit diffusion and use this to update rhs
	  //Use new rhs to update volt2
	  dsolvetridiag(n); 
	  for(i=0;i<n;i++)
	    { 
	      if(Tc[i][j]!=3)
		{
		  func=0.5*tstep*(paux[i][j]-daux[i][j]*Cauxint[i][j]);
		  Caux2[i][j]=rhs[i]+func;
		}
	    }
	}
    }

  //bereken de gemiddelde auxine conc per cell
  for(q=0;q<totcells;q++)
    MeanCells[q]=0;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Cell[i][j]!=-1)
	  {
	    q=Cell[i][j];
	    MeanCells[q]+=Caux1[i][j];
	  }
      }
  for(q=0;q<totcells;q++)
    MeanCells[q]=MeanCells[q]*InvCellSizes[q];
  
  //final update for next timestep
  //doe dit pas na het berekenen van het 
  //gemiddelde auxin level per cell
  //hierdoor doe je genexpressie update
  //nog op basis van oude aux1, net als
  //fluxes, diffusie, decay etc.    
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]!=3)
	  Caux1[i][j]=Caux2[i][j];
      }



  
  //check symmetry auxin levels
#ifndef TROPIC
  static int counter=0;

  for(j=0;j<m;j++)
    for(i=0;i<=(n/2)-1;i++)
      {
	if(  (Tc[i][j]!=3 && (counter2 ==0 )) )
	  {
	    if((int)(1000*(Caux1[i][j]-Caux1[n-1-i][j]))!=0)
	      {
		printf("something is wrong in auxin levels at %i %i %i (%.10f %i %i) and  %i %i %i(%.10f %i %i)\n",i,j,Cell[i][j],Caux1[i][j],(int)(10000*Caux1[i][j]),Tc[i][j],n-1-i,j,Cell[n-1-i][j],Caux1[n-1-i][j],(int)(10000*Caux1[n-1-i][j]),Tc[n-1-i][j]);
        MakeGraphics(t);
		counter2+=1;
		//exit(1);
	      }
	      else 	 {}

	  }
      }
  counter++;
#endif
  
}
