/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten and Kirsten ten Tusscher


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "Arrays.hh"
#include "Auxin_params.hh"
#include "Basic_params.hh"
#include "Functions.hh"
#include "Param_config.hh"
#include "Plant_layout.hh"

#include "dSFMT.h"

dsfmt_t dsfmt;

double gauss[10000];
double tempcumgauss[100000];
char dirname[10000];
int actcells;
int counter2;
double phase;

//OscillationZone can go?
//IncrAuxin and DiffAuxin both needed?
//BaselineAuxin and BaseMeanCell both needed?


int idlist[3000];
int idcounter;
int memid;
int maxmzheight;
int rightsize;
int leftsize;
int totalrootcellnrL;
int totalrootcellnrR;
//grid based properties
double **Caux1;
double **Caux2;
double **Cauxint;
double **Plasmodesmatapattern;
int **Tc;
int **Tmt;
int **Tct;
double **PinPrePat;
double **Pin2PrePat;
double **Cpin1M;
double **Cpin2M;
double **Aux1PrePat;
double **Aux1M;
double **Pltsgrid;
double **Pltsgrid2;
double **dummypict;
double *BaseMeanCell;
double *TempAuxin;
double *IncrAuxin;
//cells on grid
int **Cell;
//cell based properties
bool *Auxminpassed;
bool *OldAuxminpassed;
bool *ARRpassed;
bool *OldARRpassed;
int *CellMinI;
int *CellMaxI;
int *CellMinJ;
int *CellMaxJ;
int *CellWidth;
int *CellHeight;
int *CellSizes;
double *InvCellSizes;
double *MeanCells;
double *OldMeanCells;
double *BaselineAuxine;
double *DiffAuxine;
double *TimeOfAuxineIncrease;

//cell based gene expression
double *ARF;
double *AA;
double *BB;
double *CC;
double *DD;
double *Plt;
double *Pltexpr;
double *Plts;
double *Pltsexpr;
double *PP;
double *Pin;
double *Pin2;
double *Aux1;
double *Yuc3;
double *ARR1;
double *ARR1RGA;
double *ARR12;
double *Gh317;
double *Gh33;
double *divTF;
double *elongTF;
double *TempdivTF;
double *TempelongTF;


double *OldARF;
double *OldAA;
double *OldBB;
double *OldCC;
double *OldDD;
double *OldPlt;
double *OldPltexpr;
double *OldPlts;
double *OldPltsexpr;
double *OldPP;
double *OldPin;
double *OldPin2;
double *OldAux1;
double *OldYuc3;
double *OldARR1;
double *OldARR1RGA;
double *OldARR12;
double *OldGh317;
double *OldGh33;
double *OlddivTF;
double *OldelongTF;
double *TempOlddivTF;
double *TempOldelongTF;



//cell based growth properties
int **OldSit;
int *MapCellNrs;
int *JustChecked;
int *TimePrevGrowth;
int *CellZonationState;
int *CellDivNr;
int *CellTimeLastDiv;
int *Oscilationzone;
double *Diff;
double *Rdiv;
double *Relong;
double *Rgrow;
double *Rgrowavg;
int *OldJustChecked;
int *OldTimePrevGrowth;
int *OldCellZonationState;
int *OldCellDivNr;
int *OldCellTimeLastDiv;
int *CellsInOZ;
int *OldCellsInOZ;
double *OldDiff;
double *OldRdiv;
double *OldRelong;
double *OldRgrow;
double *OldRgrowavg;
double *RandNr;


//auxin dynamics
double *a;
double *b;
double *c;
double *rhs;
double *res;
double **paux;
double **daux;
double ***Inf;
double ***Eff;
double ***DiffAuxC;
//graphics
unsigned char *RGBdata;





void Start(int argc, char **argv)
{
  char s[1000];

  if(argc<2)
    {
      printf("usage: exc dirname\n");
      exit(1);
    }
  else
    {
      strcpy(dirname,argv[1]);
    }
}


void MakeArrays()
{
  int i,j,k;

  //1D arrays:
  if( (CellMinI=new int[totcells])== NULL) 
    { printf("error in allocating CellMinI\n"); exit(1); }
  for(i=0;i<totcells;i++)
    CellMinI[i]=0;
  if( (CellMaxI=new int[totcells])== NULL) 
    { printf("error in allocating CellMaxI\n"); exit(1); }
  for(i=0;i<totcells;i++)
    CellMaxI[i]=0;
  if( (CellMinJ=new int[totcells])== NULL) 
    { printf("error in allocating CellMinJ\n"); exit(1); }
  for(i=0;i<totcells;i++)
    CellMinJ[i]=0;
  if( (CellMaxJ=new int[totcells])== NULL) 
    { printf("error in allocating CellMaxJ\n"); exit(1); }
  for(i=0;i<totcells;i++)
    CellMaxJ[i]=0;
  if( (CellWidth=new int[totcells])== NULL) 
    { printf("error in allocating CellWidth\n"); exit(1); }
  for(i=0;i<totcells;i++)
    CellWidth[i]=0;
  if( (CellHeight=new int[totcells])== NULL) 
    { printf("error in allocating CellHeight\n"); exit(1); }
  for(i=0;i<totcells;i++)
    CellHeight[i]=0;
  if( (CellSizes=new int[totcells])== NULL) 
    { printf("error in allocating CellSizes\n"); exit(1); }
  for(i=0;i<totcells;i++)
    CellSizes[i]=0;
  if( (InvCellSizes=new double[totcells])== NULL) 
    { printf("error in allocating InvCellSizes\n"); exit(1); }
  for(i=0;i<totcells;i++)
    InvCellSizes[i]=0;
  if( (MeanCells=new double[totcells])== NULL) 
    { printf("error in allocating MeanCells\n"); exit(1); }
  for(i=0;i<totcells;i++)
    MeanCells[i]=0;
  if( (OldMeanCells=new double[totcells])== NULL) 
    { printf("error in allocating OldMeanCells\n"); exit(1); }
  for(i=0;i<totcells;i++)
  OldMeanCells[i]=0;
  if( (BaselineAuxine=new double[totcells])== NULL) 
    { printf("error in allocating BaselineAuxine\n"); exit(1); }
  for(i=0;i<totcells;i++)
  BaselineAuxine[i]=0;
  if( (DiffAuxine=new double[totcells])== NULL) 
    { printf("error in allocating DiffAuxine\n"); exit(1); }
  for(i=0;i<totcells;i++)
  DiffAuxine[i]=0;
  if( (TimeOfAuxineIncrease=new double[totcells])== NULL) 
    { printf("error in allocating TimeOfAuxineIncrease\n"); exit(1); }
  for(i=0;i<totcells;i++)
    TimeOfAuxineIncrease[i]=0;
  
  if( (Auxminpassed=new bool[totcells])== NULL) 
    { printf("error in allocating Auxminpassed\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Auxminpassed[i]=false;  
  if( (OldAuxminpassed=new bool[totcells])== NULL) 
    { printf("error in allocating OldAuxminpassed\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldAuxminpassed[i]=false;  
   if( (ARRpassed=new bool[totcells])== NULL) 
    { printf("error in allocating ARRpassed\n"); exit(1); }
  for(i=0;i<totcells;i++)
    ARRpassed[i]=false;   
   if( (OldARRpassed=new bool[totcells])== NULL) 
    { printf("error in allocating OldARRpassed\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldARRpassed[i]=false;   
  
  
  if( (ARF=new double[totcells])== NULL) 
    { printf("error in allocating ARF\n"); exit(1); }
  for(i=0;i<totcells;i++)
    ARF[i]=0;
  if( (OldARF=new double[totcells])== NULL) 
    { printf("error in allocating OldARF\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldARF[i]=0;
  if( (AA=new double[totcells])== NULL) 
    { printf("error in allocating AA\n"); exit(1); }
  for(i=0;i<totcells;i++)
    AA[i]=0;
  if( (OldAA=new double[totcells])== NULL) 
    { printf("error in allocating OldAA\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldAA[i]=0;
  if( (BB=new double[totcells])== NULL) 
    { printf("error in allocating BB\n"); exit(1); }
  for(i=0;i<totcells;i++)
    BB[i]=0;
  if( (OldBB=new double[totcells])== NULL) 
    { printf("error in allocating OldBB\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldBB[i]=0;
  if( (CC=new double[totcells])== NULL) 
    { printf("error in allocating CC\n"); exit(1); }
  for(i=0;i<totcells;i++)
    CC[i]=0;
  if( (OldCC=new double[totcells])== NULL) 
    { printf("error in allocating OldCC\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldCC[i]=0;
  if( (DD=new double[totcells])== NULL) 
    { printf("error in allocating DD\n"); exit(1); }
  for(i=0;i<totcells;i++)
    DD[i]=0;
  if( (OldDD=new double[totcells])== NULL) 
    { printf("error in allocating OldDD\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldDD[i]=0;
  if( (Plt=new double[totcells])== NULL) 
    { printf("error in allocating Plt\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Plt[i]=0;
  if( (OldPlt=new double[totcells])== NULL) 
    { printf("error in allocating OldPlt\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldPlt[i]=0;
  if( (Pltexpr=new double[totcells])== NULL) 
    { printf("error in allocating Pltexpr\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Pltexpr[i]=0;
  if( (OldPltexpr=new double[totcells])== NULL) 
    { printf("error in allocating OldPltexpr\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldPltexpr[i]=0;
  if( (Plts=new double[totcells])== NULL) 
    { printf("error in allocating Plts\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Plts[i]=0;
  if( (OldPlts=new double[totcells])== NULL) 
    { printf("error in allocating OldPlts\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldPlts[i]=0;
  if( (Pltsexpr=new double[totcells])== NULL) 
    { printf("error in allocating Pltsexpr\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Pltsexpr[i]=0;
  if( (OldPltsexpr=new double[totcells])== NULL) 
    { printf("error in allocating OldPltsexpr\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldPltsexpr[i]=0;
  if( (PP=new double[totcells])== NULL) 
    { printf("error in allocating PP\n"); exit(1); }
  for(i=0;i<totcells;i++)
    PP[i]=0;
  if( (OldPP=new double[totcells])== NULL) 
    { printf("error in allocating OldPP\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldPP[i]=0;
  if( (Pin=new double[totcells])== NULL) 
    { printf("error in allocating Pin\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Pin[i]=0;
  if( (OldPin=new double[totcells])== NULL) 
    { printf("error in allocating OldPin\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldPin[i]=0;
  if( (Pin2=new double[totcells])== NULL) 
    { printf("error in allocating Pin2\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Pin2[i]=0;
  if( (OldPin2=new double[totcells])== NULL) 
    { printf("error in allocating OldPin2\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldPin2[i]=0;
  if( (Aux1=new double[totcells])== NULL) 
    { printf("error in allocating Aux1\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Aux1[i]=0;
  if( (OldAux1=new double[totcells])== NULL) 
    { printf("error in allocating OldAux1\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldAux1[i]=0;

  if( (Yuc3=new double[totcells])== NULL) 
    { printf("error in allocating Yuc3\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Yuc3[i]=0;
  if( (OldYuc3=new double[totcells])== NULL) 
    { printf("error in allocating OldYuc3\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldYuc3[i]=0;
    
  if( (ARR1=new double[totcells])== NULL) 
    { printf("error in allocating ARR1\n"); exit(1); }
  for(i=0;i<totcells;i++)
    ARR1[i]=0;
  if( (OldARR1=new double[totcells])== NULL) 
    { printf("error in allocating OldARR1\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldARR1[i]=0;
  if( (ARR12=new double[totcells])== NULL) 
    { printf("error in allocating ARR12\n"); exit(1); }
  for(i=0;i<totcells;i++)
    ARR12[i]=0;
  if( (OldARR12=new double[totcells])== NULL) 
    { printf("error in allocating OldARR12\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldARR12[i]=0;    
  if( (ARR1RGA=new double[totcells])== NULL) 
    { printf("error in allocating ARR1RGA\n"); exit(1); }
  for(i=0;i<totcells;i++)
    ARR1RGA[i]=0;
  if( (OldARR1RGA=new double[totcells])== NULL) 
    { printf("error in allocating OldARR1RGA\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldARR1RGA[i]=0;    

  if( (Gh317=new double[totcells])== NULL) 
    { printf("error in allocating Gh317\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Gh317[i]=0;
  if( (OldGh317=new double[totcells])== NULL) 
    { printf("error in allocating OldGh317\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldGh317[i]=0;

  if( (Gh33=new double[totcells])== NULL) 
    { printf("error in allocating Gh33\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Gh33[i]=0;
  if( (OldGh33=new double[totcells])== NULL) 
    { printf("error in allocating OldGh33\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldGh33[i]=0;

    
  if( (divTF=new double[totcells])== NULL) 
    { printf("error in allocating divTF\n"); exit(1); }
  for(i=0;i<totcells;i++)
    divTF[i]=0;
  if( (OlddivTF=new double[totcells])== NULL) 
    { printf("error in allocating OlddivTF\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OlddivTF[i]=0;
  if( (elongTF=new double[totcells])== NULL) 
    { printf("error in allocating elongTF\n"); exit(1); }
  for(i=0;i<totcells;i++)
    elongTF[i]=0;
  if( (OldelongTF=new double[totcells])== NULL) 
    { printf("error in allocating OldelongTF\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldelongTF[i]=0;

  if( (TempdivTF=new double[totcells])== NULL) 
    { printf("error in allocating TempdivTF\n"); exit(1); }
  for(i=0;i<totcells;i++)
    TempdivTF[i]=0;
  if( (TempOlddivTF=new double[totcells])== NULL) 
    { printf("error in allocating TempOlddivTF\n"); exit(1); }
  for(i=0;i<totcells;i++)
    TempOlddivTF[i]=0;
  if( (TempelongTF=new double[totcells])== NULL) 
    { printf("error in allocating TempelongTF\n"); exit(1); }
  for(i=0;i<totcells;i++)
    TempelongTF[i]=0;
  if( (TempOldelongTF=new double[totcells])== NULL) 
    { printf("error in allocating TempOldelongTF\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldelongTF[i]=0;


  if( (MapCellNrs=new int[totcells])== NULL) 
    { printf("error in allocating MapCellNrs\n"); exit(1); }
  for(i=0;i<totcells;i++)
    MapCellNrs[i]=0;
  if( (JustChecked=new int[totcells])== NULL) 
    { printf("error in allocating JustChecked\n"); exit(1); }
  for(i=0;i<totcells;i++)
    JustChecked[i]=0;
  if( (OldJustChecked=new int[totcells])== NULL) 
    { printf("error in allocating OldJustChecked\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldJustChecked[i]=0;
  if( (TimePrevGrowth=new int[totcells])== NULL) 
    { printf("error in allocating TimePrevGrowth\n"); exit(1); }
  for(i=0;i<totcells;i++)
    TimePrevGrowth[i]=0;
  if( (OldTimePrevGrowth=new int[totcells])== NULL) 
    { printf("error in allocating OldTimePrevGrowth\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldTimePrevGrowth[i]=0;
  if( (CellZonationState=new int[totcells])== NULL) 
    { printf("error in allocating CellZonationState\n"); exit(1); }
  for(i=0;i<totcells;i++)
    CellZonationState[i]=0;
  if( (CellDivNr=new int[totcells])== NULL) 
    { printf("error in allocating CellDivNr\n"); exit(1); }
  for(i=0;i<totcells;i++)
    CellDivNr[i]=0;
  if( (CellTimeLastDiv=new int[totcells])== NULL) 
    { printf("error in allocating CellTimeLastDiv\n"); exit(1); }
  for(i=0;i<totcells;i++)
    CellTimeLastDiv[i]=0;
  if( (OldCellZonationState=new int[totcells])== NULL) 
    { printf("error in allocating OldCellZonationState\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldCellZonationState[i]=0;
   if( (OldCellDivNr=new int[totcells])== NULL) 
    { printf("error in allocating OldCellDivNr\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldCellDivNr[i]=0;
  if( (OldCellTimeLastDiv=new int[totcells])== NULL) 
    { printf("error in allocating OldCellTimeLastDiv\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldCellTimeLastDiv[i]=0;
  if( (CellsInOZ=new int[totcells])== NULL) 
    { printf("error in allocating CellsInOZ\n"); exit(1); }  
  for(i=0;i<totcells;i++)
	CellsInOZ[i]=0; 
  if( (OldCellsInOZ=new int[totcells])== NULL) 
    { printf("error in allocating OldCellsInOZ\n"); exit(1); }  
  for(i=0;i<totcells;i++)
	OldCellsInOZ[i]=0; 
  if( (Diff=new double[totcells])== NULL) 
    { printf("error in allocating Diff\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Diff[i]=0;
  if( (OldDiff=new double[totcells])== NULL) 
    { printf("error in allocating OldDiff\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldDiff[i]=0;

  if( (Rgrow=new double[totcells])== NULL) 
    { printf("error in allocating Rgrow\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Rgrow[i]=0;
  if( (OldRgrow=new double[totcells])== NULL) 
    { printf("error in allocating OldRgrow\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldRgrow[i]=0;
  if( (Rgrowavg=new double[totcells])== NULL) 
    { printf("error in allocating Rgrowavg\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Rgrowavg[i]=0;
  if( (OldRgrowavg=new double[totcells])== NULL) 
    { printf("error in allocating OldRgrowavg\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldRgrowavg[i]=0;

   if( (Rdiv=new double[totcells])== NULL) 
    { printf("error in allocating Rdiv\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Rdiv[i]=0;
  if( (OldRdiv=new double[totcells])== NULL) 
    { printf("error in allocating OldRdiv\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldRdiv[i]=0;
  if( (Relong=new double[totcells])== NULL) 
    { printf("error in allocating Relong\n"); exit(1); }
  for(i=0;i<totcells;i++)
    Relong[i]=0;
  if( (OldRelong=new double[totcells])== NULL) 
    { printf("error in allocating OldRelong\n"); exit(1); }
  for(i=0;i<totcells;i++)
    OldRelong[i]=0;

  if( (RandNr=new double[m])== NULL) 
    { printf("error in allocating RandNr\n"); exit(1); }
  for(i=0;i<m;i++)
    RandNr[i]=0;
  if( (a=new double[m])== NULL) 
    { printf("error in allocating a\n"); exit(1); }
  for(i=0;i<m;i++)
    a[i]=0;
  if( (b=new double[m])== NULL) 
    { printf("error in allocating b\n"); exit(1); }
  for(i=0;i<m;i++)
    b[i]=0;
  if( (c=new double[m])== NULL) 
    { printf("error in allocating c\n"); exit(1); }
  for(i=0;i<m;i++)
    c[i]=0;
  if( (rhs=new double[m])== NULL) 
    { printf("error in allocating rhs\n"); exit(1); }
  for(i=0;i<m;i++)
    rhs[i]=0;
  if( (res=new double[m])== NULL) 
    { printf("error in allocating res\n"); exit(1); }
  for(i=0;i<m;i++)
    res[i]=0;
  if( (RGBdata=new unsigned char[(m*zoom)*(n*zoom)*3])== NULL) 
    { printf("error in allocating RGB\n"); exit(1); }
  for(i=0;i<((m*zoom)*(n*zoom)*3);i++)
    RGBdata[i]=0;
    
   //BaseCaux1
  if (( BaseMeanCell = new double[m]) == NULL )
    { printf("error in allocating BaseMeanCell array\n"); exit(1);}
  for(i=0;i<m;i++)
      BaseMeanCell[i]=0;
  //TempAuxin
  if (( TempAuxin= new double[m]) == NULL )
    { printf("error in allocating TempAuxin array\n"); exit(1);}
  for(i=0;i<m;i++)
      TempAuxin[i]=0;
  //IncrAuxin
  if (( IncrAuxin = new double[m]) == NULL )
  for(i=0;i<m;i++)
      IncrAuxin[i]=0;
    

  
  
  //2D arrays:
  //OldSit
  if (( OldSit = new int*[totcells]) == NULL )
    { printf("error in allocating OldSit array\n"); exit(1);}
  for ( i = 0; i < totcells; i++ )
    {
      if (( OldSit[i] = new int[3]) == NULL )
	{ printf("error in allocating OldSit array\n"); exit(1);}
    }
  for(i=0;i<totcells;i++)
    for(j=0;j<3;j++)
      OldSit[i][j]=0;
  //Caux1
  if (( Caux1 = new double*[n]) == NULL )
    { printf("error in allocating Caux1 array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Caux1[i] = new double[m]) == NULL )
	{ printf("error in allocating Caux1 array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Caux1[i][j]=0;
  //Caux2
  if (( Caux2 = new double*[n]) == NULL )
    { printf("error in allocating Caux2 array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Caux2[i] = new double[m]) == NULL )
	{ printf("error in allocating Caux2 array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Caux2[i][j]=0;
  //Cauxint
  if (( Cauxint = new double*[n]) == NULL )
    { printf("error in allocating Cauxint array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Cauxint[i] = new double[m]) == NULL )
	{ printf("error in allocating Cauxint array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Cauxint[i][j]=0;
  //Plasmodesmatapattern;
  if (( Plasmodesmatapattern = new double*[totcells]) == NULL )
    { printf("error in allocating Plasmodesmatapattern array\n"); exit(1);}
  for ( i = 0; i < totcells; i++ )
    {
      if (( Plasmodesmatapattern[i] = new double[4]) == NULL )
	{ printf("error in allocating Plasmodesmatapattern array\n"); exit(1);}
    }
  for(i=0;i<totcells;i++)
    for(j=0;j<4;j++)
      Plasmodesmatapattern[i][j]=1;
  //Tc
  if (( Tc = new int*[n]) == NULL )
    { printf("error in allocating Tc array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Tc[i] = new int[m]) == NULL )
	{ printf("error in allocating Tc array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Tc[i][j]=0;
  //Tmt
  if (( Tmt = new int*[n]) == NULL )
    { printf("error in allocating Tmt array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Tmt[i] = new int[m]) == NULL )
	{ printf("error in allocating Tmt array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Tmt[i][j]=0;
  //Tct
  if (( Tct = new int*[n]) == NULL )
    { printf("error in allocating Tct array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Tct[i] = new int[m]) == NULL )
	{ printf("error in allocating Tct array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Tct[i][j]=0;
  //PinPrePat
  if (( PinPrePat =new double*[n]) == NULL )
    { printf("error in allocating PinPrePat array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( PinPrePat[i] = new double[m]) == NULL )
	{ printf("error in allocating PinPrePat array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      PinPrePat[i][j]=0;
  //Cpin1M
  if (( Cpin1M =new double*[n]) == NULL )
    { printf("error in allocating Cpin1M array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Cpin1M[i] = new double[m]) == NULL )
	{ printf("error in allocating Cpin1M array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Cpin1M[i][j]=0;
  //Pin2PrePat
  if (( Pin2PrePat =new double*[n]) == NULL )
    { printf("error in allocating Pin2PrePat array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Pin2PrePat[i] = new double[m]) == NULL )
	{ printf("error in allocating Pin2PrePat array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Pin2PrePat[i][j]=0;
  //Cpin2M
  if (( Cpin2M =new double*[n]) == NULL )
    { printf("error in allocating Cpin2M array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Cpin2M[i] = new double[m]) == NULL )
	{ printf("error in allocating Cpin2M array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Cpin2M[i][j]=0;


  //Aux1PrePat
  if (( Aux1PrePat =new double*[n]) == NULL )
    { printf("error in allocating Aux1PrePat array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Aux1PrePat[i] = new double[m]) == NULL )
	{ printf("error in allocating Aux1PrePat array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Aux1PrePat[i][j]=0;
  //Aux1M
  if (( Aux1M =new double*[n]) == NULL )
    { printf("error in allocating Aux1M array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Aux1M[i] = new double[m]) == NULL )
	{ printf("error in allocating Aux1M array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Aux1M[i][j]=0;
  //Pltsgrid
  if (( Pltsgrid =new double*[n]) == NULL )
    { printf("error in allocating Pltsgrid array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Pltsgrid[i] = new double[m]) == NULL )
	{ printf("error in allocating Pltsgrid array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Pltsgrid[i][j]=0;
   //Pltsgrid2
  if (( Pltsgrid2 =new double*[n]) == NULL )
    { printf("error in allocating Pltsgrid2 array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Pltsgrid2[i] = new double[m]) == NULL )
	{ printf("error in allocating Pltsgrid2 array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
    Pltsgrid2[i][j]=0;
  //dummypict
  if (( dummypict =new double*[n]) == NULL )
    { printf("error in allocating dummypict array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( dummypict[i] = new double[m]) == NULL )
	{ printf("error in allocating dummypict array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
    dummypict[i][j]=0;
  



  //Cell
  if (( Cell = new int*[n]) == NULL )
    { printf("error in allocating Cell array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( Cell[i] = new int[m]) == NULL )
	{ printf("error in allocating Cell array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Cell[i][j]=0;
  //paux
  if (( paux = new double*[n]) == NULL )
    { printf("error in allocating paux array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( paux[i] = new double[m]) == NULL )
	{ printf("error in allocating paux array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      paux[i][j]=0;
  //daux
  if (( daux = new double*[n]) == NULL )
    { printf("error in allocating daux array\n"); exit(1);}
  for ( i = 0; i < n; i++ )
    {
      if (( daux[i] = new double[m]) == NULL )
	{ printf("error in allocating daux array\n"); exit(1);}
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      daux[i][j]=0;
  
 
  //3D arrays
  //Inf
  if (( Inf= new double**[n]) == NULL )
    { printf("error in allocating Inf array\n"); exit(1);}
  for(i = 0; i < n; i++) { 
    if (( Inf[i]= new double*[m]) == NULL )
      { printf("error in allocating Inf array\n"); exit(1);}
    for(j=0; j < m; j++){
       if (( Inf[i][j]= new double[4]) == NULL )
	 { printf("error in allocating Inf array\n"); exit(1);}
    }
  }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      for(k=0;k<4;k++)
	Inf[i][j][k]=0;	
  //Eff
  if (( Eff= new double**[n]) == NULL )
    { printf("error in allocating Eff array\n"); exit(1);}
  for(i = 0; i < n; i++) { 
    if (( Eff[i]= new double*[m]) == NULL )
      { printf("error in allocating Eff array\n"); exit(1);}
    for(j=0; j < m; j++){
       if (( Eff[i][j]= new double[4]) == NULL )
	 { printf("error in allocating Eff array\n"); exit(1);}
    }
  }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      for(k=0;k<4;k++)
	Eff[i][j][k]=0;
  //DiffAuxC
  if (( DiffAuxC= new double**[n]) == NULL )
    { printf("error in allocating DiffAuxC array\n"); exit(1);}
  for(i = 0; i < n; i++) { 
    if (( DiffAuxC[i]= new double*[m]) == NULL )
      { printf("error in allocating DiffAuxC array\n"); exit(1);}
    for(j=0; j < m; j++){
       if (( DiffAuxC[i][j]= new double[4]) == NULL )
	 { printf("error in allocating DiffAuxC array\n"); exit(1);}
    }
  }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      for(k=0;k<4;k++)
	DiffAuxC[i][j][k]=0;
}



void FreeArrays()
{
  int i,j;

  //1D arrays
  delete[] CellMinI;
  delete[] CellMaxI;
  delete[] CellMinJ;
  delete[] CellMaxJ;
  delete[] CellWidth;
  delete[] CellHeight;
  delete[] CellSizes;
  delete[] InvCellSizes;
  delete[] MeanCells;
  delete[] OldMeanCells;
  delete[] BaselineAuxine;
  delete[] DiffAuxine;
  delete[] TimeOfAuxineIncrease;
  delete[] ARF;
  delete[] OldARF;
  delete[] AA;
  delete[] OldAA;
  delete[] BB;
  delete[] OldBB;
  delete[] CC;
  delete[] OldCC;
  delete[] DD;
  delete[] OldDD;
  delete[] Plt;
  delete[] OldPlt;
  delete[] Pltexpr;
  delete[] OldPltexpr;
  delete[] Plts;
  delete[] OldPlts;
  delete[] Pltsexpr;
  delete[] OldPltsexpr;
  delete[] PP;
  delete[] OldPP;
  delete[] Pin;
  delete[] OldPin;
  delete[] Pin2;
  delete[] OldPin2;
  delete[] Aux1;
  delete[] OldAux1;
  delete[] Yuc3;
  delete[] OldYuc3;
  delete[] ARR1;
  delete[] OldARR1;
  delete[] ARR1RGA;
  delete[] OldARR1RGA;
  delete[] ARR12;
  delete[] OldARR12;
  delete[] Gh317;
  delete[] OldGh317;
  delete[] Gh33;
  delete[] OldGh33;
  delete[] divTF;
  delete[] OlddivTF;
  delete[] elongTF;
  delete[] OldelongTF;
  
  delete[] TempdivTF;
  delete[] TempOlddivTF;
  delete[] TempelongTF;
  delete[] TempOldelongTF;



  delete[] MapCellNrs;
  delete[] JustChecked;
  delete[] OldJustChecked; 
  delete[] TimePrevGrowth;
  delete[] OldTimePrevGrowth;
  delete[] CellZonationState;
  delete[] OldCellZonationState;
  delete[] CellDivNr;
  delete[] OldCellDivNr;
  delete[] CellTimeLastDiv;
  delete[] OldCellTimeLastDiv;
  delete[] CellsInOZ;
  delete[] OldCellsInOZ;
  delete[] Diff;
  delete[] OldDiff;
  delete[] Rgrow;
  delete[] OldRgrow;
  delete[] Rgrowavg;
  delete[] OldRgrowavg;
  delete[] Rdiv;
  delete[] OldRdiv;
  delete[] Relong;
  delete[] OldRelong;
  delete[] RandNr;
  delete[] a;
  delete[] b;
  delete[] c;
  delete[] rhs;
  delete[] res;
  delete[] RGBdata;
  delete[] BaseMeanCell;
  delete[] IncrAuxin;
  delete[] TempAuxin;
  delete[] Auxminpassed;
  delete[] OldAuxminpassed;
  delete[] ARRpassed;
  delete[] OldARRpassed;
  
  //delete[] Oscilationzone;
  
  //2D arrays
  for(i = 0; i < totcells;i++)
    delete[] OldSit[i];
  delete[] OldSit;
  for(i = 0; i < n; i++)
    delete[] Caux1[i];
  delete[] Caux1;
  for(i = 0; i < n; i++)
    delete[] Caux2[i];
  delete[] Caux2;
  for(i = 0; i < n; i++)
    delete[] Cauxint[i];
  delete[] Cauxint;
  for( i = 0; i < totcells; i++)
    delete[] Plasmodesmatapattern[i];
  delete[] Plasmodesmatapattern;
  for(i = 0; i < n; i++)
    delete[] Tc[i];
  delete[] Tc;
  for(i = 0; i < n; i++)
    delete[] Tmt[i];
  delete[] Tmt;
  for(i = 0; i < n; i++)
    delete[] Tct[i];
  delete[] Tct;
  for(i = 0; i < n; i++)
    delete[] PinPrePat[i];
  delete[] PinPrePat;
  for(i = 0; i < n; i++)
    delete[] Cpin1M[i];
  delete[] Cpin1M;
    delete[] Pin2PrePat[i];
  delete[] Pin2PrePat;
  for(i = 0; i < n; i++)
    delete[] Cpin2M[i];
  delete[] Cpin2M;
  for(i = 0; i < n; i++)
    delete[] Aux1PrePat[i];
  delete[] Aux1PrePat;
  for(i = 0; i < n; i++)
    delete[] Aux1M[i];
  delete[] Aux1M;
  for(i = 0; i < n; i++)
    delete[] Pltsgrid[i];
  delete[] Pltsgrid;
  for(i = 0; i < n; i++)
    delete[] Pltsgrid2[i];
  delete[] Pltsgrid2;
  for(i = 0; i < n; i++)
    delete[] dummypict[i];
  delete[] dummypict;

  for(i = 0; i < n; i++)
    delete[] Cell[i];
  delete[] Cell;
  for(i = 0; i < n; i++)
    delete[] paux[i];
  delete[] paux;
  for(i = 0; i < n; i++)
    delete[] daux[i];
  delete[] daux;
  
  
  
  //3D arrays
  for(i=0;i<n;i++)
    {
      for(j=0;j<m;j++)
	delete[] Inf[i][j];
      delete[] Inf[i];
    }
  delete[] Inf;
  for(i=0;i<n;i++)
    {
      for(j=0;j<m;j++)
	delete[] Eff[i][j];
      delete[] Eff[i];
    }
  delete[] Eff;
  for(i=0;i<n;i++)
    {
      for(j=0;j<m;j++)
	delete[] DiffAuxC[i][j];
      delete[] DiffAuxC[i];
    }
  delete[] DiffAuxC;
}


void fillgauss()
{
  int i;
  int j;
  double ii;
  double gg;
  double min=-2.58;//-4;
  double max=2.58;//4;
  double cum=0;

  for(i=0;i<100000;i++)
    tempcumgauss[i]=0;

  for(i=0;i<100000;i++)//fill normal cum gauss array 
    {
      ii=i*0.0001-5;

      if(ii>=min && ii<=max)
	gg=(1./(sqrt(2*M_PI)))*exp(-ii*ii/2);
      else
	gg=0;
      cum+=gg*0.0001;
      tempcumgauss[i]=cum;
    }

  //correction for cutoff of gaussian at -2.54 and 2.54
  double maxcum=tempcumgauss[100000-1];
  for(i=0;i<100000;i++)
    {
      tempcumgauss[i]=tempcumgauss[i]/maxcum;
    }

  for(i=0;i<10000;i++)//derive inverted array 
    {
      ii=i*0.0001;
      j=0;
      while(tempcumgauss[j]<ii)
	j++;
      gauss[i]=j*0.0001-5;//-max;
    }
}


int main(int argc, char **argv)
{
  int time;
  int i,j,q;
  int starttime;

  counter2= 0;

  Start(argc,argv);
  MakeArrays();
  fillgauss();



  dsfmt_init_gen_rand(&dsfmt,18);

#ifndef RESTART
  starttime = 1;
  maxmzheight = yMZEZ;
  InitPlant();

  MakeGraphics(0);
  rightsize = TOPJCELLS;
  leftsize = TOPJCELLS;
  totalrootcellnrL = nrcellsstraightMZ+nrcellsEZ+nrcellsDZ+nrcellsHJ;
  totalrootcellnrR = nrcellsstraightMZ+nrcellsEZ+nrcellsDZ+nrcellsHJ;
#else
  starttime = givenstart;
  InitPlantFromFile(starttime);
  #ifdef RESTARTFROMZERO
    starttime = 0;

  #else
    starttime =starttime/tstep; 
  #endif
  MakeGraphics(starttime*tstep);
#endif

#ifndef GROWTH
  Plasmodesmatapatterning();
  maxmzsize();
#endif


#ifdef INITIALGROWTHSPREAD
  Plasmodesmatapatterning();
  maxmzsize();
  #ifndef DELAYEDGROWTH
    double initialgrowthtime = 0; //makes sure you can use initialgrowthtime even if it isn't defined
  #endif
  int lastcell = 0;
  int rownr = 0;
  for( int cellj = yLRCstraight; cellj<m;cellj++){ 
    if (Tc[disttoepi+2][cellj] ==3|| CellZonationState[Cell[disttoepi+2][cellj]]==200){//you're done going up through growing cells that need their last time shuffled
      break; 
    } 
    else if (Cell[disttoepi+2][cellj] !=1 && Cell[disttoepi+2][cellj] !=lastcell){

      int currentcell = lastcell;
      rownr++;
      int distortion = int( float(avgdivtime)/2. * float(((rownr*5%8)-4))/4. );// can be positive or negative up to a third of divtime
      TimePrevGrowth[Cell[disttoepi+2][cellj]] = initialgrowthtime+ distortion; 
      TimePrevGrowth[Cell[n-(disttoepi+2)-1][cellj]] = initialgrowthtime+ distortion; 
      for (int celli= disttoepi+1; celli < n/2; celli++){
        if( Cell[celli][cellj] !=1 && Cell[celli][cellj] !=currentcell){
          currentcell= Cell[celli][cellj];
          int flipcell = Cell[n-celli-1][cellj] ;
          TimePrevGrowth[currentcell] = initialgrowthtime+ distortion; 
          TimePrevGrowth[flipcell] = initialgrowthtime+ distortion; 
          if (CellMinJ[currentcell] <= yLRCstraight+2){
            CellDivNr[currentcell] = 0;
            CellDivNr[flipcell] = 0;
            }
          else if (CellMinJ[  Cell[CellMinI[currentcell] ]  [ CellMinJ[currentcell]-3 ]  ]<= yLRCstraight+2){
            CellDivNr[currentcell] = 1;
            CellDivNr[flipcell] = 1;
            }
          else {
            CellDivNr[currentcell] = 2;
            CellDivNr[flipcell] = 2;
            
          }
        }
      }
    }
  }
#endif


  phase=0;
  double frequency=2*M_PI/(3600*24);
  TrackCells(0);
  cout << "actually starting simulation"<<std::endl;
  for(time=starttime;time<=maxnrsteps;time++)//for(time=0;time<=(int)(24*3600/tstep);time++)//for(time=starttime;time<=maxnrsteps;time++)
    {
		
      //circadian clock
      phase=phase+tstep*frequency;
      if(phase>2*M_PI)
	phase=phase-2*M_PI;

      UpdateAuxin((int)(time*tstep));

      if(time%speedup2==0)
	{
	  UpdateGeneExpression((int)(time*tstep));
      #ifdef GROWTH
        #ifdef DELAYEDGROWTH
          if (time*tstep > initialgrowthtime){
            Growth((int)(time*tstep));
            }
        #else
          Growth((int)(time*tstep));
        #endif
      #endif
	  UpdateMembraneProteinsAndFluxes((int)(time*tstep));

	  TrackCells((int)(time*tstep));
	}
      

      
      if((int)(time*tstep)>0 && (int)(time*tstep)%50000==0)
	WritePlantToFile((int)(time*tstep));


      if(time%100==0)
	{
      SaveTotAuxin((int)(time*tstep));
	  SaveTotPlts((int)(time*tstep));
	}
     
      if (time%int(600/tstep)==0){
        TrackCellTypeSpaceTime(int(time*tstep));
        TrackPltsSpaceTime(int(time*tstep));
	    TrackAuxSpaceTime(int(time*tstep));
        Tracktotalmeasures(int(time*tstep));		  
		  
	  }
      
      if(time%tsave==0)
	{
	  MakeGraphics((int)(time*tstep));
	  SaveGradGridRes((int)(time*tstep));

	  //plot in vasc
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRCstraight);
	   SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1-26*HMZ-26);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1-24*HMZ-24);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1-22*HMZ-22);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1-20*HMZ-20);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1-18*HMZ-18);
	   SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1-16*HMZ-16);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1-14*HMZ-14);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1-12*HMZ-12);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1-10*HMZ-10);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1-8*HMZ-8);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1-6*HMZ-6);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1-4*HMZ-4);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1-2*HMZ-2);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1+2*HMZ+2);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1+8*HMZ+8);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+totsizeperi+2),yLRC1+16*HMZ+16);
	  
	  //plot in peri

	  //in pericycle small oscillations -8 to -2
	  //funny spikes for 0
	  //real spikes for 0 to +16
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRCstraight);
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1-26*HMZ-26);//101
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1-24*HMZ-24);//113
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1-22*HMZ-22);//125
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1-20*HMZ-20);//137
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1-18*HMZ-18);//149
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1-16*HMZ-16);//161
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1-14*HMZ-14);//173
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1-12*HMZ-12);//185
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1-10*HMZ-10);//197
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1-8*HMZ-8);//209
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1-6*HMZ-6);//221
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1-4*HMZ-4);//233
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1-2*HMZ-2);//245
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1);//257
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1+2*HMZ+2);//269
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1+8*HMZ+8);//305
	  SaveTemp((int)(time*tstep),(disttoepi+totsizeepi+totsizecort+totsizeendo+2),yLRC1+16*HMZ+16);//353

      SaveTempQC((int)(time*tstep));

 

	}
	  if (time == starttime+(100./tstep)){ 
        MakeGraphics(time*tstep);
      }
    }

  FreeArrays();


  return 0;
}
