/*

GrowingRoot simulates the regulation of root growth and developmental 
zonation by the interactions between phytohormone auxin, cytokinin mediated ARR1 and AR12, and the PLETHORA transcription factors
that together self organize control over meristem size in a simplified, two dimensional model of the Arabidopsis root.

Copyright 2017-2020, Jacob Pieter Rutten and Kirsten ten Tusscher


GrowingRoot is distributed under the terms of the GNU General Public 
License.


GrowingRoot is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GrowingRoot is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GrowingRoot.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "Arrays.hh"
#include "Auxin_params.hh"
#include "Basic_params.hh"
#include "Functions.hh"
#include "Param_config.hh"
#include "Plant_layout.hh"


double positivesigmoid(double inputval,double km,double maxval){
  double input2= inputval*inputval;
  double km2 = km*km;

  if (maxval==0){
    return (input2/(km2+input2));
  }
  else {
  double realign= (maxval*maxval)/(km2+maxval*maxval);
    return (input2/(km2+input2))/realign;
  }
}
double negativesigmoid(double inputval,double km,double maxval){
  double input2= inputval*inputval;
  double km2 = km*km;

  if (maxval==0){
    return 1-(input2/(km2+input2));
  }
  else {
    double realign= (maxval*maxval)/(km2+maxval*maxval);
    return max(0,1-(input2/(km2+input2))/realign);
  }
}

double positive4sigmoid(double inputval,double km,double maxval=100){
  double input4= inputval*inputval*inputval*inputval;
  double km4 = km*km*km*km;
  double maxval4 = maxval*maxval*maxval*maxval;
  if (maxval==0){
    return (input4/(km4+input4));
  }
  else{
    double realign= (maxval4)/(km4+maxval4);
    return (input4/(km4+input4))/realign;
  }
}



/*function that captures Plethora production dynamics*/
double UpdatePlts(int t, int q,double temp1,double tspeedupstep){
  double dtPlts= tspeedupstep*(-dPlts*Plts[q]); //fail safe makes sure that if not otherwise defined PLT still degrades plethora normally
  Pltsexpr[q]=0;
#ifdef JUNCTIONNOPLTS //TURNED on by default. Makes sure no PLETHORA is produced in hypocotyl junction cells. This prevents those cells from becoming a PLETHORA sink.
  if (CellZonationState[q]==200){
  dtPlts = tspeedupstep*(-dPlts*Plts[q]);
  Pltsexpr[q]=0;
  }
  else{
  #ifndef STATICPLTS; //STATICPLTS is static PLTS localisation used for initial parametrization regime
    dtPlts=tspeedupstep*(pPlts*temp1-dPlts*Plts[q]);
	 Pltsexpr[q]=pPlts/dPlts*(temp1);		   
  #endif
  #ifdef STATICPLTS; //default OFF
    int i= (CellMinI[q]+CellMaxI[q])/2;
    int j = CellMinJ[q]+1;
    while (Cell[i][j]!=q){j++;}
    if (Tct[i][j]==12){ //Plts production in the QC
      dtPlts = tspeedupstep*(pPlts*(1+auxAAweight)-dPlts*Plts[q]);
      Pltsexpr[q]= pPlts*(1+auxAAweight)/dPlts;
    }
    else if (Tct[i][j]==17&& (j > yQC-12)){ // Plts production SCN 
      dtPlts = tspeedupstep*(pPlts*(1+auxAAweight)*0.75-dPlts*Plts[q]);
      Pltsexpr[q]= pPlts*(1+auxAAweight)*0.75/dPlts;
    }
    else if (Tct[i][j]>=8 && Tct[i][j]<=11){ //vasculature
      double pltsprodreduction= (1+auxAAweight) * max(0,negativesigmoid ((CellMaxJ[q]-yQC),((HMZ*2+1)*2.5)));
      dtPlts =tspeedupstep*(pPlts*pltsprodreduction -dPlts*Plts[q]);
      Pltsexpr[q] = pPlts*pltsprodreduction/dPlts;
      }
    else if(Tct[i][j]==5){ //epidermis  
      double pltsprodreduction= (1+auxAAweight) * max(0,negativesigmoid ((CellMaxJ[q]-yQC),((HMZ*2+1)*4)));
      dtPlts =tspeedupstep*(pPlts*pltsprodreduction -dPlts*Plts[q]);
      Pltsexpr[q] = pPlts*pltsprodreduction/dPlts;
    }
    else if (Tct[i][j]<=4&& j> yLRCstraight&& (j > yQC-12)){
      double pltsprodreduction= (1+auxAAweight) * max(  0,negativesigmoid ((CellMaxJ[q]-yQC),((HMZ*2+1)*3))   );
      dtPlts =tspeedupstep*(pPlts*pltsprodreduction -dPlts*Plts[q]);
      Pltsexpr[q] = pPlts*pltsprodreduction/dPlts;
    }
    else if (Tct[i][j]<=6&& j< yLRCstraight&& (j > yQC-12)){
      double pltsprodreduction= (1+auxAAweight) * max(0,negativesigmoid ((CellMaxJ[q]-yQC),((HMZ*2+1)*1)));
      dtPlts =tspeedupstep*(pPlts*pltsprodreduction -dPlts*Plts[q]);
      Pltsexpr[q] = pPlts*pltsprodreduction/dPlts;
	}
    else if (Tct[i][j]<=7&& j< yLRCstraight&& (j > yQC-12)){
      double pltsprodreduction= (1+auxAAweight) * max(0,negativesigmoid ((CellMaxJ[q]-yQC),((HMZ*2+1)*1)));
      dtPlts =tspeedupstep*(pPlts*pltsprodreduction -dPlts*Plts[q]);
      Pltsexpr[q] = pPlts*pltsprodreduction/dPlts;
	}


  #endif
  }
#endif
#ifndef JUNCTIONNOPLTS
  #ifndef STATICPLTS;
    dtPlts=tspeedupstep*(pPlts*temp1-dPlts*Plts[q]);
    Pltsexpr[q]=pPlts/dPlts*(temp1);	
  #endif
  #ifdef STATICPLTS;
    int i= (CellMinI[q]+CellMaxI[q])/2;
    int j = CellMinJ[q]+1;
    for (j;Cell[i][j]==q;j++){}
    if (Tct[i][j]==12){ //Plts production in the QC
      dtPlts = tspeedupstep*(pPlts*(1+auxAAweight)-dPlts*Plts[q]);
      Pltsexpr[q]= pPlts*(1+auxAAweight)/dPlts;
    }
    else if (Tct[i][j]>=9 && Tct[i][j]<=11){ //vasculature
      double pltsprodreduction= (1+auxAAweight) * max(0,negativesigmoid ((CellMaxJ[q]-yLRCstraight),((HMZ*2+1)*2)));
      dtPlts =tspeedupstep(pPlts*pltsprodreduction -dPlts*Plts[q]);
      Pltsexpr[q] = pPlts*pltsprodreduction/dPlts;
      }
    else if(Tct[i][j]==5){ //epidermis  
      double pltsprodreduction= (1+auxAAweight) * max(0,negativesigmoid ((CellMaxJ[q]-yLRCstraight),((HMZ*2+1)*1)));
      dtPlts =tspeedupstep*(pPlts*pltsprodreduction -dPlts*Plts[q]);
      Pltsexpr[q] = pPlts*pltsprodreduction/dPlts;
    }
    else if (Tct[i][j]<=4&& j> yLRCstraight){
      double pltsprodreduction= (1+auxAAweight) * max(0,negativesigmoid ((CellMaxJ[q]-yLRCstraight),((HMZ*2+1)*1.5)));
      dtPlts =tspeedupstep*(pPlts*pltsprodreduction -dPlts*Plts[q]);
      Pltsexpr[q] = pPlts*pltsprodreduction/dPlts;
    }
  #endif
#endif

  return dtPlts;
}

/*function that captures possible ARR production dynamics */ 
std::pair<double,double> UpdateARRS(int t, int q,double tspeedupstep){

  double dtARR1= 0;
  double dtARR12= 0;
  #define ARRREGULATED
    #ifndef STATICARRS

      #ifdef AUTOACT
        double ARR12production = pARR12*
        positivesigmoid(MeanCells[q],AuxKM_ARR12,0) * negativesigmoid(Plts[q],PltsKM_ARR12,0);
        double ARR1production = pARR1* positivesigmoid(MeanCells[q],AuxKM_ARR1,0)  * negativesigmoid(Plts[q],PltsKM_ARR1,0);
      #else
        double ARR12production = pARR12*
        positivesigmoid(MeanCells[q],AuxKM_ARR12,0) * negativesigmoid(Plts[q],PltsKM_ARR12,0); 
     

          double gibberellineffect = 0.; //we now let ARRs autoinhibition get stronger in the presence of additional external gibberellin, to simulate its effect on RGA.
          if( t<= embryoGAtimelimit){
            gibberellineffect = 1.;
          }
          else if (t< (embryoGAtimelimit+reductiontime)){
            gibberellineffect = ( 1.0-double(t-embryoGAtimelimit)/double(reductiontime));
          }
          double ARR1production = pARR1*positivesigmoid(MeanCells[q],AuxKM_ARR1,0)  * negativesigmoid(Plts[q],PltsKM_ARR1,0) *  (1.-gibberellineffect);



      #endif
    #endif
    #ifdef STATICARRS //

      int i= (CellMinI[q]+CellMaxI[q])/2;
      int j = CellMinJ[q]+1;
      double ARR1distribution = 0;
      double ARR12distribution = 0;
      double endlower = 1;
      if (arrversion ==1){
        if (j > yLRCstraight+HMZ*25+HEZ*6){
        endlower = negativesigmoid ((CellMaxJ[q]-( yLRCstraight+HMZ*25+HEZ*6)),HDZ*2,0);
        }
      }

      if (j > yLRCstraight){
        //for (j;Cell[i][j]==q;j++){}
        if (Tct[i][j]<=5&& j> yLRCstraight){ //epidermis en lateral root cap
          if(t > embryoGAtimelimit){ 
            ARR1distribution  = positivesigmoid((CellMaxJ[q]-yLRCstraight),(HMZ)*20,0) *endlower ;
            }
          ARR12distribution = positivesigmoid((CellMaxJ[q]-yLRCstraight),(HMZ)*10,0)*endlower;
          }
        else if (Tct[i][j]==6){
          if(t > embryoGAtimelimit){
            ARR1distribution  = 0.9* positivesigmoid((CellMaxJ[q]-yLRCstraight),HMZ*20,0)*endlower ;
            }
          ARR12distribution = 0.9*positivesigmoid((CellMaxJ[q]-yLRCstraight),HMZ*10,0)*endlower;
          }
        else if (Tct[i][j]==7){
          if(t > embryoGAtimelimit){
            ARR1distribution  = 0.75*positivesigmoid((CellMaxJ[q]-yLRCstraight),HMZ*20,0)*endlower ;
            }
          ARR12distribution =  0.75*positivesigmoid((CellMaxJ[q]-yLRCstraight),HMZ*10,0)*endlower;
          }
        else if (Tct[i][j]<=11&& Tct[i][j]>=8 ){
          if(t > embryoGAtimelimit){
            ARR1distribution  = positivesigmoid((CellMaxJ[q]-yLRCstraight),HMZ*20,0)*endlower ;
            }
          ARR12distribution = positivesigmoid((CellMaxJ[q]-yLRCstraight),HMZ*10,0)*endlower;

          }
        }

      double ARR1production = pARR1* ARR1distribution;
      double ARR12production = pARR12*ARR12distribution;
    #endif

  #ifdef ARRREGULATED
    #ifdef JUNCTIONNOARR
    if (CellZonationState[q]!=200){ //you can not produce in junction when JunctionnoARR is defined.
    dtARR1 = tspeedupstep*(ARR1production
   -dARR1*ARR1[q]);
    dtARR12 =  tspeedupstep*(ARR12production
   -dARR12*ARR12[q]);
      }
    else{
      dtARR1 = tspeedupstep*(
      -dARR1*ARR1[q]);
      dtARR12 =  tspeedupstep*(
      -dARR12*ARR12[q]);
      }
    #else
    dtARR1 = tspeedupstep*(ARR1production
      -dARR1*ARR1[q]);
    dtARR12 =  tspeedupstep*(ARR12production
      -dARR12*ARR12[q]);
    #endif
  #endif

  return std::make_pair(dtARR1,dtARR12);
  

}






void UpdateGeneExpression(int t)
{
  int q,k;
  static double sat2plt=satplt*satplt;
  static double sat2pin=satpin*satpin;
  static double sat2plts=satplts*satplts;
  static double sat2yuc3=PltsKM_YUC3*PltsKM_YUC3;
  //static double sat2gh317=satgh317*satgh317;
  static double sat2aux1=AuxinKM_AUX1*AuxinKM_AUX1;

  double dtARF,dtAA,dtBB,dtCC,dtDD,dtPlt,dtPlts;
  double dtPin,dtPin2,dtAux1;             
  double dtARR1,dtARR12,dtARR1RGA,dtYuc3,dtGh317,dtGh33,dtPP,dtdivTF,dtelongTF;
  static double tspeedupstep=tstep*speedup2;
  int meani,meanj;
 
  //update gene expression levels: cell based
  for(q=0;q<actcells;q++)//for(q=0;q<totcells;q++)
    {
      dtARF=0;
      dtAA=0;
      dtBB=0;
      dtCC=0;
      dtDD=0;
      dtPlt=0;
      dtPlts=0;
      dtPP=0;
      dtPin=0;
      dtPin2=0;
      dtAux1=0;
      dtYuc3=0;
      dtARR1=0;
      dtARR1RGA=0;
      dtARR12=0;
      dtGh33=0;
      dtGh317=0;
      dtdivTF=0;
      dtelongTF=0;

      meani=(int)(0.5*(CellMinI[q]+CellMaxI[q]));
      meanj=(int)(0.5*(CellMinJ[q]+CellMaxJ[q]));



      //ARF depends on auxin
 
      
      //if((Tct[meani][meanj]==1 && CellMinJ[q]>yLRCstraight+3*HMZ+3) || (Tct[meani][meanj]>=13 && Tct[meani][meanj]<=16))
      //if(Tct[meani][meanj]==1 && CellMinJ[q]>yLRCstraight+3*HMZ+3)
      //	ass=3*0.01;
		double arfproductioncontrol = 1.;
	   #ifdef ARFLATCONTROL
        if( CellMinJ[q]>yLRCstraight+fullarfdistance && (Tct[CellMinI[q]][CellMinJ[q]]<=4)){
		    arfproductioncontrol *= (1.0-LRCARFfac)+LRCARFfac*max(0.0,(arfreductiondistance-(CellMinJ[q]-(yLRCstraight+fullarfdistance)))/arfreductiondistance);
        }
		#endif
      #ifndef ARFDIMERIZE
        dtARF=tspeedupstep*(diss*(ARFtotal*arfproductioncontrol-ARF[q])-ass*ARF[q]*(pI/(dIbasal+dITIR1*MeanCells[q])));
      #else
        dtARF=tspeedupstep*(diss*(ARFtotal*arfproductioncontrol-ARF[q])-ass*ARF[q]*(pI/(dIbasal+dITIR1*dITIR1*MeanCells[q]*MeanCells[q])));
        if (ARF[q]+dtARF <0){
           cout << " ARFS reduced to "<< ARF[q]+dtARF << "production is " <<diss*(ARFtotal*arfproductioncontrol-ARF[q]) << " loss is " << ass*ARF[q]*(pI/(dIbasal+dITIR1*dITIR1*MeanCells[q]*MeanCells[q])) << " Arf were " <<ARF[q]<<" dtarf " << dtARF <<" tspeed " <<tspeedupstep <<std::endl;
        } 
      #endif      

      //dtAA=tspeedupstep*(pGG*ARF[q]*ARF[q]/(ARF[q]*ARF[q]+ARFKM_A*ARFKM_A )-dGG*AA[q]);
       dtAA=tspeedupstep*(pGG * positivesigmoid(ARF[q],ARFKM_A)-dGG*AA[q]);

      dtBB=tspeedupstep*(pGG*positivesigmoid(AA[q],AKM_B)-dGG*BB[q]);
      #ifndef ABDONLY
        dtCC=tspeedupstep*(pGG*positivesigmoid(BB[q],BKM_C)-dGG*CC[q]);
        dtDD=tspeedupstep*(pGG*positivesigmoid(CC[q],CKM_D)-dGG*DD[q]);
      #endif
      #ifdef ABDONLY
        dtCC =tspeedupstep*( -dGG*CC[q]);
        dtDD=tspeedupstep*(pGG*positivesigmoid(BB[q],CKM_D)-dGG*DD[q]);
      #endif


    double temp1=1.0;

	{
    //#define PLTSDIRECTARF
    #ifdef PLTSDIRECTARF
      double satpltarf = 55;
      temp1=positivesigmoid(DD[q],satplts)*positivesigmoid(ARF[q],satpltarf);
    #else
	  temp1=positivesigmoid(DD[q],satplts);
    #endif 
    }
	#ifdef AUTOACT
    double temp2=1.0;

	temp2*= positivesigmoid( Plts[q],halfAAPlts,(pPlts*(1+auxAAweight)/dPlts) );
   #ifdef NEWAUTO
     temp2*= auxAAweight*positive4sigmoid(MeanCells[q],halfAuxAA,0);
   #else
     temp2*= auxAAweight*positive4sigmoid(ARF[q],halfARFAA);
   #endif
	//temp2*= auxAAweight*(ARF[q]*ARF[q]*ARF[q]*ARF[q]/(halfARFAA*halfARFAA*halfARFAA*halfARFAA+ARF[q]*ARF[q]*ARF[q]*ARF[q]));
	temp1 = temp1+temp2;
	#endif
#define ARR1PLTINHIBITION

#ifdef ARR1PLTINHIBITION
    
    #ifndef ARR1INHIBIT
      double ARR1inhibit = 1.0;
    #endif 

      dtARR1RGA= pARR1bind * ((1.0-ARR1_self_max_inhibit)+(ARR1_self_max_inhibit* negativesigmoid(ARR1RGA[q],ARR1KM_self_inhibit) ))  * ARR1[q]*RGAARR1boundeff - pARR1bind *ARR1RGA[q];
      double ARR1effect = ARR1RGA[q];	
    #ifdef ARRCOMBINEDPLTINHIBITION
      temp1*= negativesigmoid(ARR1effect+ARR12[q],ARRsKM_Plts,0);
    #else
      temp1*= ((1.0-ARR1pltinhibitfrac)+ ARR1pltinhibitfrac* negativesigmoid(ARR1effect,ARR1KM_Plts,100*RGAARR1boundeff));
    #endif 
    #ifndef RGAARR1EFFECT
      temp1*= ((1.0-ARR1pltinhibitfrac)+ ARR1pltinhibitfrac *negativesigmoid(ARR1[q],ARR1KM_Plts));
    #endif 
    
#endif
 
#define ARR12PLTINHIBITION
#ifdef ARR12PLTINHIBITION
  #ifndef ARRCOMBINEDPLTINHIBITION
    temp1*= (1.0-ARR12pltinhibitfrac)+ ARR12pltinhibitfrac *negativesigmoid(ARR12[q],ARR12KM_Plts); 
  #endif
#endif

dtPlts = UpdatePlts(t,q,temp1,tspeedupstep);


#ifdef RGAARR1EFFECT
  double localARR1= ARR1RGA[q]; //ARR1effect;
#endif
#ifndef RGAARR1EFFECT
    double localARR1 = ARR1[q];
#endif

     
#ifdef PINREGULATED 

    double pin1prodreg = 1.0;
    double pin2prodreg = 1.0;
  #define PINARRREGULATED
  #ifdef PINARRREGULATED

    #ifdef JOINEDARRPINEFFS
      if (CellZonationState[q]>50 && MeanCells[q]>200){

      } 
      pin1prodreg *= (1.0)-PINARRSfac* (positivesigmoid(localARR1+ARR12[q],ARRSKMPINS,0)*(1.0-positivesigmoid(MeanCells[q],AuxinKM_PIN1SHY2,0)) );
      #ifdef PIN2ARRREGULATED
      pin2prodreg *= (1.0)-PINARRSfac* (positivesigmoid(localARR1+ARR12[q],ARRSKMPINS,0)*(1.0-positivesigmoid(MeanCells[q],AuxinKM_PIN2SHY2,0)) );
      #endif
    #else
      pin1prodreg *= 1.0- min(1,(( PINARR1fac*positivesigmoid(localARR1,ARR1KMPINS,100*RGAARR1boundeff)) + (PINARR12fac* positivesigmoid(ARR12[q],ARR12KMPINS))   *(1.0-positivesigmoid(MeanCells[q],AuxinKM_PIN1SHY2,0))));

      #ifdef PIN2ARRREGULATED
      pin2prodreg *= 1.0-  min(1, (( PINARR1fac*positivesigmoid(localARR1,ARR1KMPINS,100*RGAARR1boundeff) + PINARR12fac* positivesigmoid(ARR12[q],ARR12KMPINS))*(1.0-positivesigmoid(MeanCells[q],AuxinKM_PIN2SHY2,0)))) ;
      #endif

    #endif

  #endif

      pin1prodreg*= (1.0-pin1Pltsfac)+pin1Pltsfac*positivesigmoid(Plts[q],PltsKM_PIN1,0);
      pin2prodreg*= (1.0-pin2Pltsfac)+pin2Pltsfac*negativesigmoid(Plts[q],PltsKM_PIN2,0);


     #ifdef pin1auxindownregulated
 
       pin1prodreg *= negativesigmoid(MeanCells[q],down_AuxinKM_PIN1SHY2,0);
     #endif
	  double pPIN1 = pPIN;
     double dPIN1 = dPIN;
     #ifndef pPINSEPARATE
     double pPIN2 = pPIN;
     double dPIN1 = dPIN;
     double dPIN2 = dPIN;
     #endif //else it is already defined in header


     dtPin=tspeedupstep*(pin1prodreg*pPIN1-dPIN1*Pin[q]);
	  dtPin2=tspeedupstep*(pin2prodreg*pPIN2-dPIN2*Pin2[q]);



#endif

      //Pin,Yuc3,Gh33, all depend on pltslow
#ifdef YUC3REGULATED
  if (CellZonationState[q]==200) //makes sure you do not produce extra auxin in non meristemic region upon initialization
    {
    dtYuc3=tspeedupstep*(-dYUC3*Yuc3[q]);
    }
  else{
   #ifdef AUTOACT
      dtYuc3=tspeedupstep*(pYUC3*positivesigmoid(Plts[q],PltsKM_YUC3,(pPlts*(1+auxAAweight)/dPlts))-dYUC3*Yuc3[q]);
   #else
      dtYuc3=tspeedupstep*(pYUC3*positivesigmoid(Plts[q],PltsKM_YUC3,0)-dYUC3*Yuc3[q]);
   #endif
  }

#endif
#ifdef GH33REGULATED
  if (CellZonationState[q]==200) //makes sure you do not produce extra auxin in non meristemic region upon initialization
    {
    dtGh33=tspeedupstep*(-dGH33*Gh33[q]);
    }
  else{
    dtGh33=tspeedupstep*(pGH33*negativesigmoid(Plts[q],PltsKM_GH33,0)-dGH33*Gh33[q]);
    }
#endif 


#ifdef AUX1REGULATED
      //double auxindep=(MeanCells[q]*MeanCells[q]/(MeanCells[q]*MeanCells[q]+(ckfac*AuxinKM_AUX1)*(ckfac*AuxinKM_AUX1)));
      double auxindep=positivesigmoid(MeanCells[q],AuxinKM_AUX1,0);
  #ifndef AUX1PARTREGGED
  double aux1_regfac = 1.;
  #endif
      dtAux1=tspeedupstep*(pAUX1*( (1.-aux1_regfac)+ aux1_regfac*(auxindep) )-dAUX1*Aux1[q]);
#endif

  std::pair<double,double> ARRS = UpdateARRS(t,q,tspeedupstep);
  dtARR1 = ARRS.first ;
  dtARR12= ARRS.second;



#ifdef GH317EPIDERMISONLY
  int iicheck=(CellMinI[q]);
  int jjcheck = ((CellMinJ[q]+CellMaxJ[q]+1)/2);
  int qqq = Cell[iicheck][jjcheck];
  while (qqq !=q){
	iicheck+=1;
    qqq = Cell[iicheck][jjcheck];
    }
    
  if (Tct[iicheck][jjcheck]<0 ||Tct[iicheck][jjcheck]>5 ||CellMinJ[q]<yLRCstraight ){ //all celltypes below 5 are either lateral root cap
     dtGh317  =tspeedupstep*(-dGH317*Gh317[q]);

  }
  else
#endif
  {
   double gh317regulated = 1.0;
    gh317regulated = positivesigmoid(localARR1+ARR12[q],ARRsKM_GH317,0);
    dtGh317=tspeedupstep*(pGH317*gh317regulated-dGH317*Gh317[q]);
  }


  #ifndef ARR1DIVFRAC  
  double ARRdivfrac = 1.;
  #endif
  dtdivTF =  tspeedupstep*(pDiv * positivesigmoid(Plts[q],PltsKM_Div,0) * ((1.-ARRdivfrac) +ARRdivfrac*negativesigmoid(ARR1[q],ARR1KM_Div,0)) - dDiv * divTF[q]);

  #ifdef ARRELONGPLUSSED
    #ifdef ARREFFECTPLUS
      double ARRelongeffect = (localARR1+ARR12[q]);
    #else
      double ARRelongeffect = (ARR1[q]+ARR12[q]);
    #endif
  #else
    #ifndef ARRELONGEFFECTEMAX
    double ARRelongeffect = max(ARR1[q],ARR12[q]);
    #else
    double ARRelongeffect = max(localARR1,ARR12[q]);
    #endif
  #endif
  double diminishingElongation= negativesigmoid(Plts[q],PltsKM_El,0);

  #ifndef BASEARRELONGFRAC
  double baseARRelongfrac = 1;
  #endif
  #ifdef DIVREPRESSELONG
  double divelongrepression =  maxdivelong * (1.0 -  ((divTF[q]*divTF[q]*divTF[q]*divTF[q])/(divTF[q]*divTF[q]*divTF[q]*divTF[q]+DivKM_El*DivKM_El*DivKM_El*DivKM_El) ) ) ;
    dtelongTF =  tspeedupstep*(pEl *(  divelongrepression+  baseElong * diminishingElongation+ (1.-baseElong-maxdivelong)*(positivesigmoid(ARRelongeffect,ARRsKM_El,0) )   ) - dEl * elongTF[q]);
  #else
  dtelongTF =  tspeedupstep*(pEl *(    baseElong * diminishingElongation+ (1.-baseElong)*((1-baseARRelongfrac)+baseARRelongfrac*positivesigmoid(ARRelongeffect,ARRsKM_El,0) )   ) - dEl * elongTF[q]);
  #endif


   ARF[q]+=dtARF;
   AA[q]+=dtAA;
   BB[q]+=dtBB;
   CC[q]+=dtCC;
   DD[q]+=dtDD;
   Plt[q]+=dtPlt;
   Plts[q]+=dtPlts;
   PP[q]+=dtPP;
   Pin[q]+=dtPin;
   Pin2[q]+=dtPin2;
   Aux1[q]+=dtAux1;

   ARR1[q]+=dtARR1;
   ARR12[q]+=dtARR12;
   ARR1RGA[q]+=dtARR1RGA;
   Yuc3[q]+=dtYuc3;
   Gh317[q]+=dtGh317;
   Gh33[q]+=dtGh33;
   divTF[q]+=dtdivTF;
   elongTF[q]+=dtelongTF;

  }

#ifdef PLASMODESMATAOPEN
  int i,j,ii,jj;
  double pdflux=Pltsdiff*double(tspeedupstep)/(double)xstep;
  double verticalPDflux = pdflux;
  double lateralPDflux = pdflux; // largely redundant since the plasmodesmata are all patterned to accomodate differences.
  CellularDiffusion(Plts,verticalPDflux,lateralPDflux,1,0.0,tspeedupstep);
#endif
}

